### Analysis run with R version 3.4.3 (2017-11-30) and platform x86_64-apple-darwin15.6.0
### Running under: macOS  10.14.5 ###
### File to run inferential statistics for Lutscher et al. (2019) ###
### At Home and Abroad: The Use of Denial-of-Service Attacks ###
### during Elections in Non-democratic Regimes ###

### File should be run top-down as later operations depend on before specified functions ###

##Set working directory##
wd <- "" #Change to folder where R files and dosel_final.csv is stored
setwd(wd)
rm(list = ls())

##Load packages
library(data.table) #Version 1.12.0
library(MASS) #Version 7.3-50
library(multiwayvcov) #Version 1.2.3
library(lmtest) #Version 0.9-36
library(plm) #Version 1.7-0
library(splines) #Version 3.4.3 
library(VGAM) #Version 1.0-5
library(texreg) #Version 1.36.23

dosel<-as.data.table(read.csv("dosel_final.csv"))
dosel$yearweek<-as.Date(dosel$yearweek)
dosel$X<-NULL

#v2x_polyarchy = inverted electoral democracy index by V-dem project

##### Main results #####

## Attacks on the country ##

plm1 <- plm(log(number_dos_unf+1)~election_period_eg,
            model="within",effect="individual",index=c("yearcountry"), data=dosel)

pdwtest(plm1) #Durbin-Watson test for positive AR(1) corr.

pbgtest(plm1) #Breusch-Godfrey tests for higher-order autocorrelation

#Breusch-Pagan test for heteroscedasticity
bptest(log(number_dos_unf+1) ~ election_period_eg+ factor(yearcountry), data = dosel, studentize=F)

#Correct for serial correlation and clusters/heteroscedasticity: Adjust for that with HAC correction (Newey-West S.E.)
se1<-coeftest(plm1,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

#Add interaction terms
plm2 <- plm(log(number_dos_unf+1)~election_period_eg+election_period_eg:v2x_polyarchy,
            model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2 <- vcovNW(plm2,type="HC1",cluster="group",maxlag=3)
se2<-coeftest(plm2,vcov_se2)


## Attacks on foreign hosts ##

#Subset data (2014 - 2016)
dosel_out<- as.data.table(dosel[year>=2014])

plm1o <- plm(log(number_dos_outside+1)~election_period_eg,
             model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
se1o<-coeftest(plm1o,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))


#Add interaction terms
plm2o <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy,
             model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2o <- vcovNW(plm2o,type="HC1",cluster="group",maxlag=3)
se2o<-coeftest(plm2o,vcov_se2o)


##### Report main models (Table 2) #####

library(texreg)

extract.plm <- function(model,var) {
  s <- summary(model)
  names <- rownames(s$coef)
  co <- s$coef[, 1]
  se <- var[, 2]
  pval <- var[, 4]
  
  n <- nobs(model)
  
  gof <- c(n)
  
  gof.names <- c("Num.\\ obs.")
  
  tr <- createTexreg(
    coef.names = names,
    coef = co,
    se = se,
    pvalues = pval,
    gof.names = gof.names,
    gof = gof
  )
  return(tr)
}

models <- mapply(extract.plm,list(plm1,plm2,plm1o,plm2o),list(se1,se2,se1o,se2o))

#Main results (Table 2)
result <- texreg(models,digits=3,
                 custom.coef.map=list("election_period_eg" = "Election period",
                                      "election_period_eg:v2x_polyarchy" = "Election period $\\\ times$ autocracy index"),
                custom.note = "", caption = "Relationship between election periods dependent on the level of autocracy and DoS attacks (country/week).
                Note: Robust standard errors clustered at the country-year level in parentheses. $^{***}p<0.001$, $^{**}p<0.01$, $^*p<0.05$}",
                label = "tab:main_results")


##### Plot main models (Figure 4) #####

#Simulation of transformed coefficient (%) for election period (=1) dependent on autocracy index (0-1)

sim_marginal <- function(model,vcov,data,terms){
  temp.data1 <- data.frame(v2x_polyarchy =seq(min(data$v2x_polyarchy,na.rm=T),max(data$v2x_polyarchy,na.rm=T),by=0.005),election_period =1)
  bootsamples1<- mvrnorm(10000,coef(model),vcov)
  dim(bootsamples1)
  latent1 <- 1:nrow(temp.data1)
  if (terms==1){
    for(i in 1:dim(bootsamples1)[1]){
      print(i)
      latent1 <-   cbind(latent1, (exp(bootsamples1[i,names(coef(model))[1]]*temp.data1[,"election_period"]+
                                         bootsamples1[i,names(coef(model))[2]]*temp.data1[,"election_period"]*temp.data1[,"v2x_polyarchy"])-1)*100)
      
    }
    upper1 <- apply(latent1[,2:ncol(latent1)],1,quantile,.975)
    lower1 <- apply(latent1[,2:ncol(latent1)],1,quantile,.025)
    mean1 <- apply(latent1[,2:ncol(latent1)],1,mean)
    
    sims <- list(mean1,lower1,upper1,temp.data1)
    return(sims)}
  if (terms==2){
    for(i in 1:dim(bootsamples1)[1]){
      print(i)
      latent1 <-   cbind(latent1,(exp(bootsamples1[i,names(coef(model))[1]]*temp.data1[,"election_period"]+
                                        bootsamples1[i,names(coef(model))[2]]*temp.data1[,"election_period"]*temp.data1[,"v2x_polyarchy"]+
                                        bootsamples1[i,names(coef(model))[3]]*temp.data1[,"election_period"]*I(temp.data1[,"v2x_polyarchy"])^2
      )-1)*100) #Take exponential and - 1 to transform coefficient to % impact on log(DoS attacks)
    }
    upper1 <- apply(latent1[,2:ncol(latent1)],1,quantile,.975)
    lower1 <- apply(latent1[,2:ncol(latent1)],1,quantile,.025)
    mean1 <- apply(latent1[,2:ncol(latent1)],1,mean)
    
    sims <- list(mean1,lower1,upper1,temp.data1)
    return(sims)
  }
}

sims1 <- sim_marginal(plm2,vcov_se2,dosel,terms=1)
sims2 <- sim_marginal(plm2o,vcov_se2o,dosel_out,terms=1)


#Attacks on the country
par(mfrow=c(1,2))
plot(sims1[[4]][,"v2x_polyarchy"],sims1[[1]],type="l",xlim=c(min(dosel$v2x_polyarchy,na.rm=T)
                                                             ,max(dosel$v2x_polyarchy,na.rm=T)),ylim=c(-10,10),
     xlab="Autocracy index",ylab="Coefficient for election period (%)",main="DoS attacks on domestic hosts\n (Model 2)")
lines(sims1[[4]][,"v2x_polyarchy"],sims1[[3]],lty=2)
lines(sims1[[4]][,"v2x_polyarchy"],sims1[[2]],lty=2)
abline(h=0) 
rug(model.frame(plm2)[,"v2x_polyarchy"])

#Attacks on foreign hosts
plot(sims2[[4]][,"v2x_polyarchy"],sims2[[1]],type="l",xlim=c(min(dosel_out$v2x_polyarchy,na.rm=T)
                                                             ,max(dosel_out$v2x_polyarchy,na.rm=T)),ylim=c(-15,20),
     xlab="Autocracy index",ylab="",main="DoS attacks on foreign hosts\n (Model 4)")
lines(sims2[[4]][,"v2x_polyarchy"],sims2[[3]],lty=2)
lines(sims2[[4]][,"v2x_polyarchy"],sims2[[2]],lty=2)
abline(h=0) 
rug(model.frame(plm2o)[,"v2x_polyarchy"])

dev.print(pdf,"./Figures/attacks_both.pdf",width=9, height=5)


##### Different bandwitdh: 2 weeks, 1 week, only election week (Table 3) #####

results <- function(var1){
  plm1_temp <- plm(log(number_dos_unf+1)~get(var1),
                   model="within",effect="individual",index=c("yearcountry"), data=dosel)
  se1_temp<-coeftest(plm1,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))
  
  plm2_temp <- plm(log(number_dos_unf+1)~get(var1)+get(var1):v2x_polyarchy,
                   model="within",effect="individual",index=c("yearcountry"), data=dosel)
  vcov_se2_temp <- vcovNW(plm2_temp,type="HC1",cluster="group",maxlag=3)
  se2_temp<-coeftest(plm2_temp,vcov_se2_temp)
  
  #Outside data (2014 - 2016)
  
  plm1o_temp <- plm(log(number_dos_outside+1)~get(var1),
                    model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
  se1o_temp <- coeftest(plm1o_temp,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))
  
  plm2o_temp <- plm(log(number_dos_outside+1)~get(var1)+get(var1):v2x_polyarchy,
                    model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
  vcov_se2o_temp <- vcovNW(plm2o_temp,type="HC1",cluster="group",maxlag=3)
  se2o_temp <-coeftest(plm2o_temp,vcov_se2o_temp)
  
  #Return results
  
  models <- mapply(extract.plm,list(plm1_temp,plm2_temp,plm1o_temp,plm2o_temp),
                   list(se1_temp,se2_temp,se1o_temp,se2o_temp))
  #Main results
  
  if (var1 == "election_period_eg1"){
    result <- texreg(models,digits=3,
                     custom.coef.map=list("get(var1)" = "Election period","get(var1):v2x_polyarchy" = "Election period x autocracy index"))
  }
  if (var1 == "election_period_eg2"){
    result <- texreg(models,digits=3,
                     custom.coef.map=list("get(var1)" = "Election period","get(var1):v2x_polyarchy" = "Election period x autocracy index"))
  }
  if (var1 == "election_period_eg6"){
    result <- texreg(models,digits=3,
                     custom.coef.map=list("get(var1)" = "Election period","get(var1):v2x_polyarchy" = "Election period x autocracy index",
                                          "get(var1):I(v2x_polyarchy^2)" = "Election period x autocracy index (sq.)"))
  }
  if (var1 == "election_eg"){
    result <- texreg(models,digits=3,
                     custom.coef.map=list("get(var1)" = "Election week","get(var1):v2x_polyarchy" = "Election week x autocracy index"))
  }
  return(result)
  
}

result_2 <- results("election_period_eg2")

result_1 <- results("election_period_eg1")

result_week <- results("election_eg")

#Additional results show smaller coefficient when period is extended (not reported in paper)
result_6 <- results("election_period_eg6")

##### 1/Time to nearest election (Table A.2) #####
#Election week is set to 1.
dosel$election_eg_time <- ifelse(dosel$election_eg==1,0,dosel$election_eg_time)
dosel_out$election_eg_time <- ifelse(dosel_out$election_eg==1,0,dosel_out$election_eg_time)

dosel$election_x <- 1/(dosel$election_eg_time+1)
dosel_out$election_x <- 1/(dosel_out$election_eg_time+1)

plm1n <- plm(log(number_dos_unf+1)~election_x,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1n<-coeftest(plm1n,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2n <- plm(log(number_dos_unf+1)~election_x +election_x:v2x_polyarchy,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2n <- vcovNW(plm2n,type="HC1",cluster="group",maxlag=3)
se2n<-coeftest(plm2n,vcov_se2n)

#Outside data (2014 - 2016)
plm1on <- plm(log(number_dos_outside+1)~election_x,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
se1on<-coeftest(plm1on,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2on <- plm(log(number_dos_outside+1)~election_x+election_x:v2x_polyarchy,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2on <- vcovNW(plm2on,type="HC1",cluster="group",maxlag=3)
se2on<-coeftest(plm2on,vcov_se2on)

#Results
models_time <- mapply(extract.plm,list(plm1n,plm2n,plm1on,plm2on),list(se1n,se2n,se1on,
                                                                       se2on))

result_time <- texreg(models_time,digits=3,
                      custom.coef.map=list("election_x" = "Temporal proximity",
                                           "election_x:v2x_polyarchy" = "Temporal proximity x autocracy index"))

##### Pre- and Post-election period (Table 4) #####
plm1pre <- plm(log(number_dos_unf+1)~pre_election_eg,
               model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1pre<-coeftest(plm1pre,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2pre <- plm(log(number_dos_unf+1)~pre_election_eg+pre_election_eg:v2x_polyarchy,
               model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2pre <- vcovNW(plm2pre,type="HC1",cluster="group",maxlag=3)
se2pre<-coeftest(plm2pre,vcov_se2pre)

#Outside data (2014 - 2016)
plm1opre <- plm(log(number_dos_outside+1)~pre_election_eg,
                model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
se1opre<-coeftest(plm1opre,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2opre <- plm(log(number_dos_outside+1)~pre_election_eg+pre_election_eg:v2x_polyarchy,
                model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2opre <- vcovNW(plm2opre,type="HC1",cluster="group",maxlag=3)
se2opre<-coeftest(plm2opre,vcov_se2opre)

#Results
models <- mapply(extract.plm,list(plm1pre,plm2pre,plm1opre,plm2opre),
                 list(se1pre,se2pre,se1opre,se2opre))

#Main results
result_pre <- texreg(models,digits=3,
                     custom.coef.map=list("pre_election_eg" = "Pre-election period",
                                          "pre_election_eg:v2x_polyarchy" = "Pre-election period x autocracy index"
                                         ))

#Post-election models
plm1post <- plm(log(number_dos_unf+1)~post_election_eg,
                model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1post<-coeftest(plm1post,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2post <- plm(log(number_dos_unf+1)~post_election_eg+post_election_eg:v2x_polyarchy,
                model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2post <- vcovNW(plm2post,type="HC1",cluster="group",maxlag=3)
se2post<-coeftest(plm2post,vcov_se2post)

#Outside data (2014 - 2016)
plm1opost <- plm(log(number_dos_outside+1)~post_election_eg,
                 model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
se1opost<-coeftest(plm1opost,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2opost <- plm(log(number_dos_outside+1)~post_election_eg+post_election_eg:v2x_polyarchy,
                 model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2opost <- vcovNW(plm2opost,type="HC1",cluster="group",maxlag=3)
se2opost<-coeftest(plm2opost,vcov_se2opost)

#Results
models <- mapply(extract.plm,list(plm1post,plm2post,plm1opost,plm2opost),
                 list(se1post,se2post,se1opost,se2opost))

#Main results
result_post <- texreg(models,digits=3,
                      custom.coef.map=list("post_election_eg" = "Post-election period",
                                           "post_election_eg:v2x_polyarchy" = "Post-election period x autocracy index",
                                           "post_election_eg:I(v2x_polyarchy^2)" = "Post-election period x autocracy index (sq.)"))


##### Subsets in democracies and autocracies (Table A.3) #####
plm_sub <- plm(log(number_dos_unf+1)~election_period_eg,
               model="within",effect="individual",index=c("yearcountry"), 
               data=dosel[v2x_polyarchy<0.5])
vcov_se_sub <- vcovNW(plm_sub,type="HC1",cluster="group",maxlag=3)
se_sub<-coeftest(plm_sub,vcov_se_sub)

plm_sub2 <- plm(log(number_dos_unf+1)~election_period_eg,
                model="within",effect="individual",index=c("yearcountry"), 
                data=dosel[v2x_polyarchy>=0.5])
vcov_se_sub2 <- vcovNW(plm_sub2,type="HC1",cluster="group",maxlag=3)
se_sub2<-coeftest(plm_sub2,vcov_se_sub2)

#Outside data (Subset 2014 - 2016)
plm_subo <- plm(log(number_dos_outside+1)~election_period_eg,
                model="within",effect="individual",index=c("yearcountry"), 
                data=dosel_out[v2x_polyarchy<0.5])
vcov_se_subo <- vcovNW(plm_subo,type="HC1",cluster="group",maxlag=3)
se_subo<-coeftest(plm_subo,vcov_se_subo)

plm_sub2o <- plm(log(number_dos_outside+1)~election_period_eg,
                 model="within",effect="individual",index=c("yearcountry"), 
                 data=dosel_out[v2x_polyarchy>=0.5])
vcov_se_sub2o <- vcovNW(plm_sub2o,type="HC1",cluster="group",maxlag=3)
se_sub2o<-coeftest(plm_sub2o,vcov_se_sub2o)


#Results
models_dummies<- mapply(extract.plm,list(plm_sub,plm_sub2,
                                         plm_subo,plm_sub2o)
                        ,list(se_sub,se_sub2,se_subo,se_sub2o))

#Main results
result_dummies <- texreg(models_dummies,digits=3,
                         custom.coef.map=list("election_period_eg" = "Election period"))

#Calculation of point estimate in % and confidence intervals presented on p.23
point <- (exp(plm_sub2o$coefficients)-1)*100 #point estimate in %
ci_l <- point-1.96*((exp(summary(plm_sub2o)$coefficients[,2])-1)*100) #approximation
ci_u <- point+1.96*((exp(summary(plm_sub2o)$coefficients[,2])-1)*100) #approximation


##### With Uppsala conflict data (Table A.4) #####
plm1u <- plm(log(number_dos_unf+1)~election_period_eg+log(conflict_event+1),
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1u<-coeftest(plm1u,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2u <- plm(log(number_dos_unf+1)~election_period_eg+election_period_eg:v2x_polyarchy
               +log(conflict_event+1),
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2u <- vcovNW(plm2u,type="HC1",cluster="group",maxlag=3)
se2u<-coeftest(plm2u,vcov_se2u)


#Outside attacks
plm1ou <- plm(log(number_dos_outside+1)~election_period_eg+log(conflict_event+1),
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se1ou <- vcovNW(plm1ou,type="HC1",cluster="group",maxlag=3)
se1ou<-coeftest(plm1ou,vcov_se1ou)

plm2ou <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy
              +log(conflict_event+1),
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2ou <- vcovNW(plm2ou,type="HC1",cluster="group",maxlag=3)
se2ou<-coeftest(plm2ou,vcov_se2ou)

#Report with Conflict data
#Results
models_conflict <- mapply(extract.plm,list(plm1u,plm2u,plm1ou,plm2ou),list(se1u,se2u,se1ou,
                                                                        se2ou))

#Main results
result_conflict <- texreg(models_conflict,digits=3,
                       custom.coef.map=list("election_period_eg" = "Election period",
                                            "election_period_eg:v2x_polyarchy" = "Election period x autocracy index",
                                           "log(conflict_event + 1)" = "Conflict events (ln+1)"))


##### With Time Trend (Table A.5) #####
#Count number of observations per country
dosel <- dosel[,count:=seq(.N),by=c("ccode")]

plm1t <- plm(log(number_dos_unf+1)~election_period_eg+v2x_polyarchy+
               factor(ccode):bs(count,3),
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1t<-coeftest(plm1t,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2t <- plm(log(number_dos_unf+1)~election_period_eg+election_period_eg*v2x_polyarchy+
               +factor(ccode):bs(count,3),
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2t <- vcovNW(plm2t,type="HC1",cluster="group",maxlag=3)
se2t<-coeftest(plm2t,vcov_se2t)


#Outside attacks
dosel_out <-dosel_out[,count:=seq(.N),by=c("ccode")]
plm1ot <- plm(log(number_dos_outside+1)~election_period_eg+v2x_polyarchy+factor(ccode):bs(count,3),
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se1ot <- vcovNW(plm1ot,type="HC1",cluster="group",maxlag=3)
se1ot<-coeftest(plm1ot,vcov_se1ot)

plm2ot <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy+
                factor(ccode):bs(count,3),
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2ot <- vcovNW(plm2ot,type="HC1",cluster="group",maxlag=3)
se2ot<-coeftest(plm2ot,vcov_se2ot)



#Results with time trends results
models_trend <- mapply(extract.plm,list(plm1t,plm2t,plm1ot,plm2ot),list(se1t,se2t,se1ot,
                                                                        se2ot))

result_trend <- texreg(models_trend,digits=3,
                       custom.coef.map=list("election_period_eg" = "Election period",
                                            "election_period_eg:v2x_polyarchy" = "Election period x autocracy index"))

##### Polity IV (Table A.6)#####
#First noromalize polity4 variable in the same way as V-Dem index (0,1)
dosel$polity2s <- (1-((dosel$polity2-min(dosel$polity2,na.rm = T))/
                             (max(dosel$polity2,na.rm = T)-
                                min(dosel$polity2,na.rm = T))))
dosel_out$polity2s <- (1-((dosel_out$polity2-min(dosel_out$polity2,na.rm = T))/
                                 (max(dosel_out$polity2,na.rm = T)-
                                    min(dosel_out$polity2,na.rm = T))))

plm1p <- plm(log(number_dos_unf+1)~election_period_eg,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1p<-coeftest(plm1p,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2p <- plm(log(number_dos_unf+1)~election_period_eg+election_period_eg:polity2s,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2p <- vcovNW(plm2p,type="HC1",cluster="group",maxlag=3)
se2p<-coeftest(plm2p,vcov_se2p)


#Outside attacks
plm1op <- plm(log(number_dos_outside+1)~election_period_eg,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se1op <- vcovNW(plm1op,type="HC1",cluster="group",maxlag=3)
se1op<-coeftest(plm1op,vcov_se1op)

plm2op <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:polity2s,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2op <- vcovNW(plm2op,type="HC1",cluster="group",maxlag=3)
se2op<-coeftest(plm2op,vcov_se2op)


models_pol <- mapply(extract.plm,list(plm1p,plm2p,plm1op,plm2op),list(se1p,se2p,se1op,
                                                                      se2op))

#Main results
result_pol <- texreg(models_pol,digits=3,
                     custom.coef.map=list("election_period_eg" = "Election period",
                                          "election_period_eg:polity2s" = "Election period x Polity2"))


##### Strong attacks (Table A.7) ######
plm1s <- plm(log(number_dos_large+1)~election_period_eg,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
se1s<-coeftest(plm1s,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2s <- plm(log(number_dos_large+1)~election_period_eg+election_period_eg:v2x_polyarchy,
             model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2s <- vcovNW(plm2s,type="HC1",cluster="group",maxlag=3)
se2s<-coeftest(plm2s,vcov_se2s)


#Outside attacks
plm1os <- plm(log(number_dos_outside_large+1)~election_period_eg,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se1os <- vcovNW(plm1os,type="HC1",cluster="group",maxlag=3)
se1os<-coeftest(plm1os,vcov_se1os)

plm2os <- plm(log(number_dos_outside_large+1)~election_period_eg+election_period_eg:v2x_polyarchy,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2os <- vcovNW(plm2os,type="HC1",cluster="group",maxlag=3)
se2os <-coeftest(plm2os,vcov_se2os)


models_strong <- mapply(extract.plm,list(plm1s,plm2s,plm1os,plm2os),list(se1s,se2s,se1os,
                                                                         se2os))

#Main results
result_strong <- texreg(models_strong,digits=3,
                        custom.coef.map=list("election_period_eg" = "Election period",
                                             "election_period_eg:v2x_polyarchy" = "Election period x autocracy index"
                                            ))


##### LDV approach (Table A.8) #####

for(i in 1:4){
  anscols <-paste0("number_dos_unf_lag",i)
  dosel<-dosel[,(anscols):=shift(number_dos_unf,n=i,fill=NA,type="lag"),by=ccode]
}

plm1l <- plm(log(number_dos_unf+1)~election_period_eg+log(number_dos_unf_lag1+1),
             model="within",index=c("yearcountry"), data=dosel)
pdwtest(plm1l) #Durbin Watson Test suggest no serious AR-1 correlation

pbgtest(plm1l)
#Generalized test still highlight issues with serial correlation, yet graphically inspection
#show that residuals are rather small (and pos. and neg.)
acf(plm1l$residuals)

#Cluster around country-year using HC SE
se1l<-coeftest(plm1l,vcov = function (x) vcovHC(x,type="HC1",cluster="group"))

plm2l <- plm(log(number_dos_unf+1)~election_period_eg+
               election_period_eg:v2x_polyarchy+
               log(number_dos_unf_lag1+1),
             model="within",index=c("yearcountry"), data=dosel)


vcov_se2l <- vcovHC(plm2l,type="HC1",cluster="group")
se2l<-coeftest(plm2l,vcov_se2l) 

plm3l <- plm(log(number_dos_unf+1)~election_eg+
               log(number_dos_unf_lag1+1),
             model="within",index=c("yearcountry"), data=dosel)

pdwtest(plm3l) #Durbin Watson Test suggest no serious AR-1 correlation

pbgtest(plm3l) #Generalized test still highlight issues with serial correlation, yet graphically inspection
#show that residuals are rather small (and pos. and neg.)
acf(plm3l$residuals)


vcov_se3l <- vcovHC(plm3l,type="HC1",cluster="group")
se3l<-coeftest(plm3l,vcov_se3l) 

plm4l <- plm(log(number_dos_unf+1)~election_eg+
               election_eg:v2x_polyarchy+
               log(number_dos_unf_lag1+1),
             model="within",index=c("yearcountry"), data=dosel)

vcov_se4l <- vcovHC(plm4l,type="HC1",cluster="group")
se4l<-coeftest(plm4l,vcov_se4l) 


#Outside attacks
for(i in 1:5){
  anscols <-paste0("number_dos_outside_lag",i)
  dosel_out<-dosel_out[,(anscols):=shift(number_dos_outside,n=i,fill=NA,type="lag"),by=ccode]
}

plm1ol <- plm(log(number_dos_outside+1)~election_period_eg+log(number_dos_outside_lag1+1)+log(number_dos_outside_lag2+1)+
                log(number_dos_outside_lag3+1)+log(number_dos_outside_lag4+1)+log(number_dos_outside_lag5+1),
              model="within",index=c("yearcountry"), data=dosel_out)

pdwtest(plm1ol) #Durbin Watson Test suggest no serious AR-1 correlation

pbgtest(plm1ol)
#Generalized test still highlight issues with serial correlation, yet graphically inspection
#show that there appears to be no systemtatic pattern, pos. and neg. outliers
acf(plm1ol$residuals)


vcov_se1ol <- vcovHC(plm1ol,type="HC1",cluster="group")
se1ol<-coeftest(plm1ol,vcov_se1ol)

plm2ol <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy
                +log(number_dos_outside_lag1+1)+log(number_dos_outside_lag2+1)+
                log(number_dos_outside_lag3+1)+log(number_dos_outside_lag4+1)+log(number_dos_outside_lag5+1),
              model="within",index=c("yearcountry"), data=dosel_out)

vcov_se2ol <- vcovHC(plm2ol,type="HC1",cluster="group")
se2ol<-coeftest(plm2ol,vcov_se2ol)

plm3ol <- plm(log(number_dos_outside+1)~election_eg+
               log(number_dos_outside_lag1+1),
             model="within",index=c("yearcountry"), data=dosel_out)

pdwtest(plm3ol) #Durbin Watson Test suggest no serious AR-1 correlation.

pbgtest(plm3ol)
#Generalized test still highlight issues with serial correlation, yet graphically inspection
#show that there appears to be no systemtatic pattern and rather neg. outliers
acf(plm3ol$residuals)

vcov_se3ol <- vcovHC(plm3ol,type="HC1",cluster="group")
se3ol<-coeftest(plm3ol,vcov_se3ol) 


plm4ol <- plm(log(number_dos_outside+1)~election_eg+
               election_eg:v2x_polyarchy+
               log(number_dos_outside_lag1+1),
             model="within",index=c("yearcountry"), data=dosel_out)

vcov_se4ol <- vcovHC(plm4ol,type="HC1",cluster="group")
se4ol<-coeftest(plm4ol,vcov_se4ol) 


models_lagged <- mapply(extract.plm,list(plm1l,plm2l,plm3l,plm4l,plm1ol,plm2ol,
                                         plm3ol,plm4ol),
                        list(se1l,se2l,se3l,se4l,se1ol,
                             se2ol,se3ol,se4ol))

#Report results
result_lag <- texreg(models_lagged,digits=3,
                     custom.coef.map=list(
                       "election_period_eg" = "Election period",
                       "election_period_eg:v2x_polyarchy" = "Election period x autocracy index",
                       "election_eg" = "Election week",
                       "election_eg:v2x_polyarchy" = "Election week x autocracy index"
                       
                      )
)


###### Outside attacks 2016 only & Placebotest with outside attacks (Table A.9) #####

#Outside data (Subset 2016)
dosel_robs <- as.data.table(dosel[year>=2016])

plm1os <- plm(log(number_dos_outside+1)~election_period_eg,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_robs)
se1os<-coeftest(plm1os,vcov = function (x) vcovNW(x,type="HC1",cluster="group",maxlag=3))

plm2os <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy,
              model="within",effect="individual",index=c("yearcountry"), data=dosel_robs)
vcov_se2os <- vcovNW(plm2os,type="HC1",cluster="group",maxlag=3)
se2os<-coeftest(plm2os,vcov_se2os)

#Placebo tests
plm_placebo <- plm(log(number_dos_outside_ra+1)~election_period_eg,
                   model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se_placebo <- vcovNW(plm_placebo,type="HC1",cluster="group",maxlag=3)
se_placebo<-coeftest(plm_placebo,vcov_se_placebo)

plm_placebo2 <- plm(log(number_dos_outside_ra+1)~election_period_eg+election_period_eg:v2x_polyarchy,
                    model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se_placebo2 <- vcovNW(plm_placebo2,type="HC1",cluster="group",maxlag=3)
se_placebo2 <- coeftest(plm_placebo2,vcov_se_placebo2)


#Without CDNs
plm_cdn <- plm(log(number_dos_outside_wocdn+1)~election_period_eg,
               model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se_cdn <- vcovNW(plm_cdn,type="HC1",cluster="group",maxlag=3)
se_cdn<-coeftest(plm_cdn,vcov_se_cdn)

plm_cdn2 <- plm(log(number_dos_outside_wocdn+1)~election_period_eg+election_period_eg:v2x_polyarchy,
                model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se_cdn2 <- vcovNW(plm_cdn2,type="HC1",cluster="group",maxlag=3)
se_cdn2 <- coeftest(plm_cdn2,vcov_se_cdn2)



#Report results
models_outside <- mapply(extract.plm,list(plm1os,plm2os,plm_placebo,plm_placebo2,plm_cdn,plm_cdn2),
                         list(se1os,se2os,se_placebo,se_placebo2,se_cdn,se_cdn2))

result_outside <- texreg(models_outside,digits=3,
                         custom.coef.map=list("election_period_eg" = "Election period",
                                              "election_period_eg:v2x_polyarchy" = "Election period x autocracy index"
                                              ))

###### Explorative quadratic models #####


## Attacks on the country ##

#Add interaction terms
plm2q <- plm(log(number_dos_unf+1)~election_period_eg+election_period_eg:v2x_polyarchy+
              election_period_eg:I(v2x_polyarchy^2),
            model="within",effect="individual",index=c("yearcountry"), data=dosel)
vcov_se2q <- vcovNW(plm2q,type="HC1",cluster="group",maxlag=3)
se2q <- coeftest(plm2q,vcov_se2q)

#Add interaction terms
plm2oq <- plm(log(number_dos_outside+1)~election_period_eg+election_period_eg:v2x_polyarchy+
               election_period_eg:I(v2x_polyarchy^2),
             model="within",effect="individual",index=c("yearcountry"), data=dosel_out)
vcov_se2oq <- vcovNW(plm2oq,type="HC1",cluster="group",maxlag=3)
se2oq <- coeftest(plm2oq,vcov_se2oq)


models_q <- mapply(extract.plm,list(plm2q,plm2oq ),list(se2q,se2oq))

##### Report quadratic models results (Table A.10) #####
result_q <- texreg(models_q,digits=3,
                 custom.coef.map=list("election_period_eg" = "Election period",
                                      "election_period_eg:v2x_polyarchy" = "Election period x autocracy index",
                                      "election_period_eg:I(v2x_polyarchy^2)" = "Election period x autocracy index (sq.)"))


#### Plot quadratic models (Figure 5) ####

sims1q <- sim_marginal(plm2q,vcov_se2q,dosel,terms=2)
sims2q <- sim_marginal(plm2oq,vcov_se2oq,dosel_out,terms=2)

par(mfrow=c(1,2))
plot(sims1q[[4]][,"v2x_polyarchy"],sims1q[[1]],type="l",xlim=c(min(dosel$v2x_polyarchy,na.rm=T)
                                                             ,max(dosel$v2x_polyarchy,na.rm=T)),ylim=c(-15,40),
     xlab="Autocracy index",ylab="Coefficient of election period (%)",main="DoS attacks on domestic hosts\n (Model A.10.1)")
lines(sims1q[[4]][,"v2x_polyarchy"],sims1q[[3]],lty=2)
lines(sims1q[[4]][,"v2x_polyarchy"],sims1q[[2]],lty=2)
abline(h=0) 
rug(model.frame(plm2q)[,"v2x_polyarchy"])

#Attacks on foreign hosts
plot(sims2q[[4]][,"v2x_polyarchy"],sims2q[[1]],type="l",xlim=c(min(dosel_out$v2x_polyarchy,na.rm=T)
                                                             ,max(dosel_out$v2x_polyarchy,na.rm=T)),ylim=c(-20,12),
     xlab="Autocracy index",ylab="",main="DoS attacks on foreign hosts\n (Model A.10.2)")
lines(sims2q[[4]][,"v2x_polyarchy"],sims2q[[3]],lty=2)
lines(sims2q[[4]][,"v2x_polyarchy"],sims2q[[2]],lty=2)
abline(h=0) 
rug(model.frame(plm2oq)[,"v2x_polyarchy"])

dev.print(pdf,"./Figures/attacks_quad.pdf",width=9, height=5)
